<?php

declare(strict_types=1);

namespace Drupal\decoupled_json_log\Plugin\jsonrpc\Method;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\decoupled_json_log\Exception\DecoupledJsonLogException;
use Drupal\jsonrpc\Exception\JsonRpcException;
use Drupal\jsonrpc\MethodInterface;
use Drupal\jsonrpc\Object\Error;
use Drupal\jsonrpc\Object\ParameterBag;
use Drupal\jsonrpc\Plugin\JsonRpcMethodBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Webmozart\Assert\Assert;

/**
 * Creates a JSON log entity.
 *
 * @JsonRpcMethod(
 *   id = "decoupled_json_log.create",
 *   usage = @Translation("Logs an event."),
 *   access = {"create log_json"},
 *   params = {
 *      "bundle" = @JsonRpcParameterDefinition(
 *        schema={"type"="string"},
 *        required=true
 *      ),
 *     "label" = @JsonRpcParameterDefinition(
 *       schema={"type"="string"},
 *       required=true
 *     ),
 *     "log" = @JsonRpcParameterDefinition(
 *       schema={"type"="string"},
 *       required=true
 *     ),
 *     "deviceInfo" = @JsonRpcParameterDefinition(
 *       schema={"type"="string"},
 *       required=false
 *     ),
 *   }
 * ),
 */
final class CreateLogJson extends JsonRpcMethodBase {

  /**
   * The current user account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected AccountInterface $currentUser;

  /**
   * Logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected LoggerChannelInterface $loggerFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Constructs a new EntityResourceBase object.
   *
   * @param array $configuration
   *   JSON-RPC config.
   * @param string $plugin_id
   *   The plugin ID.
   * @param \Drupal\jsonrpc\MethodInterface $plugin_definition
   *   The plugin definition.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current user.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The Watchdog logger.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(array $configuration,
  string $plugin_id,
  MethodInterface $plugin_definition,
  AccountInterface $account,
  LoggerChannelFactoryInterface $logger_factory,
  EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->currentUser = $account;
    $this->loggerFactory = $logger_factory->get('decoupled_json_log');
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): CreateLogJson {
    if (!$plugin_definition instanceof MethodInterface) {
      throw new DecoupledJsonLogException('Plugin definition should be MethodInterface!');
    }
    return new self(
      $configuration, $plugin_id, $plugin_definition,
      $container->get('current_user'),
      $container->get('logger.factory'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function execute(ParameterBag $params) {
    try {
      $event_to_log = $params->get('log');
      Assert::stringNotEmpty($event_to_log, 'Log must be a non-empty string!');

      $device_info = $params->get('deviceInfo');
      $bundle = $params->get('bundle');
      Assert::stringNotEmpty($bundle, 'Bundle must be a non-empty string!');
      $label = $params->get('label');
      Assert::stringNotEmpty($label, 'Label must be a non-empty string!');

      // @todo Sanitize $event_to_log.
      $log_json_entity = $this->entityTypeManager->getStorage('log_json')->create([
        'uid' => $this->currentUser->id(),
        'bundle' => $bundle,
        'label' => $label,
        'log' => $event_to_log,
        'device_info' => $device_info,
      ]);
      return boolval($log_json_entity->save());
    }
    /* @phpstan-ignore-next-line Thrown exceptions in catch block must bundle previous exception. */
    catch (\LogicException $e) {
      $error_message = $e->getMessage();
      $this->loggerFactory->error($error_message);
      $error = Error::internalError();
      throw JsonRpcException::fromError($error);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function outputSchema(): array {
    return ['type' => 'boolean'];
  }

}
